@@ -3,6 +3,7 @@ use ruff_python_ast::{self as ast, Expr};
 use ruff_python_stdlib::identifiers::{is_identifier, is_mangled_private};
 use ruff_source_file::LineRanges;
 use ruff_text_size::Ranged;
+use unicode_normalization::UnicodeNormalization;
 
 use crate::checkers::ast::Checker;
 use crate::fix::edits::pad;
@@ -29,6 +30,21 @@ use crate::{AlwaysFixableViolation, Edit, Fix};
 /// obj.foo
 /// ```
 ///
+/// ## Fix safety
+/// The fix is marked as unsafe for attribute names that are not in NFKC (Normalization Form KC)
+/// normalization. Python normalizes identifiers using NFKC when using attribute access syntax
+/// (e.g., `obj.attr`), but does not normalize string arguments passed to `getattr`. Rewriting
+/// `getattr(obj, "ſ")` to `obj.ſ` would be interpreted as `obj.s` at runtime, changing behavior.
+///
+/// For example, the long s character `"ſ"` normalizes to `"s"` under NFKC, so:
+/// ```python
+/// # This accesses an attribute with the exact name "ſ" (if it exists)
+/// value = getattr(obj, "ſ")
+///
+/// # But this would normalize to "s" and access a different attribute
+/// obj.ſ  # This is interpreted as obj.s, not obj.ſ
+/// ```
+///
 /// ## References
 /// - [Python documentation: `getattr`](https://docs.python.org/3/library/functions.html#getattr)
 #[derive(ViolationMetadata)]
@@ -69,8 +85,14 @@ pub(crate) fn getattr_with_constant(checker: &Checker, expr: &Expr, func: &Expr,
         return;
     }
 
+    // Mark fixes as unsafe for non-NFKC attribute names. Python normalizes identifiers using NFKC, so using
+    // attribute syntax (e.g., `obj.attr`) would normalize the name and potentially change
+    // program behavior.
+    let attr_name = value.to_str();
+    let is_unsafe = attr_name.nfkc().collect::<String>() != attr_name;
+
     let mut diagnostic = checker.report_diagnostic(GetAttrWithConstant, expr.range());
-    diagnostic.set_fix(Fix::safe_edit(Edit::range_replacement(
+    let edit = Edit::range_replacement(
         pad(
             if matches!(
                 obj,
@@ -88,5 +110,11 @@ pub(crate) fn getattr_with_constant(checker: &Checker, expr: &Expr, func: &Expr,
             checker.locator(),
         ),
         expr.range(),
-    )));
+    );
+    let fix = if is_unsafe {
+        Fix::unsafe_edit(edit)
+    } else {
+        Fix::safe_edit(edit)
+    };
+    diagnostic.set_fix(fix);
 }
