@@ -4,7 +4,7 @@ use crate::comments::format::{
 use crate::expression::maybe_parenthesize_expression;
 use crate::expression::parentheses::{Parentheses, Parenthesize};
 use crate::prelude::*;
-use crate::statement::clause::{ClauseHeader, clause_body, clause_header};
+use crate::statement::clause::{ClauseHeader, clause};
 use crate::statement::stmt_class_def::FormatDecorators;
 use crate::statement::suite::SuiteKind;
 use ruff_formatter::write;
@@ -60,12 +60,13 @@ impl FormatNodeRule<StmtFunctionDef> for FormatStmtFunctionDef {
                     decorators: decorator_list,
                     leading_definition_comments,
                 },
-                clause_header(
+                clause(
                     ClauseHeader::Function(item),
-                    trailing_definition_comments,
                     &format_with(|f| format_function_header(f, item)),
+                    trailing_definition_comments,
+                    body,
+                    SuiteKind::Function,
                 ),
-                clause_body(body, SuiteKind::Function, trailing_definition_comments),
             ]
         )?;
 
@@ -93,6 +94,7 @@ impl FormatNodeRule<StmtFunctionDef> for FormatStmtFunctionDef {
 fn format_function_header(f: &mut PyFormatter, item: &StmtFunctionDef) -> FormatResult<()> {
     let StmtFunctionDef {
         range: _,
+        node_index: _,
         is_async,
         decorator_list: _,
         name,
