@@ -1,7 +1,8 @@
-use crate::{completion, find_node::covering_node};
+use crate::completion;
 
 use ruff_db::{files::File, parsed::parsed_module};
 use ruff_diagnostics::Edit;
+use ruff_python_ast::find_node::covering_node;
 use ruff_text_size::TextRange;
 use ty_project::Db;
 use ty_python_semantic::create_suppression_fix;
@@ -29,12 +30,11 @@ pub fn code_actions(
 
     let mut actions = Vec::new();
 
-    // Suggest imports for unresolved references (often ideal)
-    // TODO: suggest qualifying with an already imported symbol
+    // Suggest imports/qualifications for unresolved references (often ideal)
     let is_unresolved_reference =
         lint_id == LintId::of(&UNRESOLVED_REFERENCE) || lint_id == LintId::of(&UNDEFINED_REVEAL);
     if is_unresolved_reference
-        && let Some(import_quick_fix) = create_import_symbol_quick_fix(db, file, diagnostic_range)
+        && let Some(import_quick_fix) = unresolved_fixes(db, file, diagnostic_range)
     {
         actions.extend(import_quick_fix);
     }
@@ -49,7 +49,7 @@ pub fn code_actions(
     actions
 }
 
-fn create_import_symbol_quick_fix(
+fn unresolved_fixes(
     db: &dyn Db,
     file: File,
     diagnostic_range: TextRange,
@@ -59,7 +59,7 @@ fn create_import_symbol_quick_fix(
     let symbol = &node.expr_name()?.id;
 
     Some(
-        completion::missing_imports(db, file, &parsed, symbol, node)
+        completion::unresolved_fixes(db, file, &parsed, symbol, node)
             .into_iter()
             .map(|import| QuickFix {
                 title: import.label,
@@ -84,6 +84,7 @@ mod tests {
         system::{DbWithWritableSystem, SystemPathBuf},
     };
     use ruff_diagnostics::Fix;
+    use ruff_python_trivia::textwrap::dedent;
     use ruff_text_size::{TextRange, TextSize};
     use ty_project::ProjectMetadata;
     use ty_python_semantic::{
@@ -149,15 +150,14 @@ mod tests {
 
         assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @r"
         info[code-action]: Ignore 'unresolved-reference' for this line
-         --> main.py:2:17
+         --> main.py:2:5
           |
-        2 |             b = a / 0  # ty:ignore[division-by-zero]
-          |                 ^
+        2 | b = a / 0  # ty:ignore[division-by-zero]
+          |     ^
           |
         1 | 
-          -             b = a / 0  # ty:ignore[division-by-zero]
-        2 +             b = a / 0  # ty:ignore[division-by-zero, unresolved-reference]
-        3 |
+          - b = a / 0  # ty:ignore[division-by-zero]
+        2 + b = a / 0  # ty:ignore[division-by-zero, unresolved-reference]
         ");
     }
 
@@ -171,15 +171,14 @@ mod tests {
 
         assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @r"
         info[code-action]: Ignore 'unresolved-reference' for this line
-         --> main.py:2:17
+         --> main.py:2:5
           |
-        2 |             b = a / 0  # ty:ignore[division-by-zero,]
-          |                 ^
+        2 | b = a / 0  # ty:ignore[division-by-zero,]
+          |     ^
           |
         1 | 
-          -             b = a / 0  # ty:ignore[division-by-zero,]
-        2 +             b = a / 0  # ty:ignore[division-by-zero, unresolved-reference]
-        3 |
+          - b = a / 0  # ty:ignore[division-by-zero,]
+        2 + b = a / 0  # ty:ignore[division-by-zero, unresolved-reference]
         ");
     }
 
@@ -193,15 +192,14 @@ mod tests {
 
         assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @r"
         info[code-action]: Ignore 'unresolved-reference' for this line
-         --> main.py:2:17
+         --> main.py:2:5
           |
-        2 |             b = a / 0  # ty:ignore[division-by-zero   ]
-          |                 ^
+        2 | b = a / 0  # ty:ignore[division-by-zero   ]
+          |     ^
           |
         1 | 
-          -             b = a / 0  # ty:ignore[division-by-zero   ]
-        2 +             b = a / 0  # ty:ignore[division-by-zero, unresolved-reference   ]
-        3 |
+          - b = a / 0  # ty:ignore[division-by-zero   ]
+        2 + b = a / 0  # ty:ignore[division-by-zero, unresolved-reference   ]
         ");
     }
 
@@ -215,15 +213,14 @@ mod tests {
 
         assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @r"
         info[code-action]: Ignore 'unresolved-reference' for this line
-         --> main.py:2:17
+         --> main.py:2:5
           |
-        2 |             b = a / 0  # ty:ignore[division-by-zero] some explanation
-          |                 ^
+        2 | b = a / 0  # ty:ignore[division-by-zero] some explanation
+          |     ^
           |
         1 | 
-          -             b = a / 0  # ty:ignore[division-by-zero] some explanation
-        2 +             b = a / 0  # ty:ignore[division-by-zero] some explanation  # ty:ignore[unresolved-reference]
-        3 |
+          - b = a / 0  # ty:ignore[division-by-zero] some explanation
+        2 + b = a / 0  # ty:ignore[division-by-zero] some explanation  # ty:ignore[unresolved-reference]
         ");
     }
 
@@ -241,22 +238,22 @@ mod tests {
 
         assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @r"
         info[code-action]: Ignore 'unresolved-reference' for this line
-         --> main.py:3:21
+         --> main.py:3:9
           |
-        2 |               b = (
-        3 | /                     a  # ty:ignore[division-by-zero]
-        4 | |                     /
-        5 | |                     0
-          | |_____________________^
-        6 |               )
+        2 |   b = (
+        3 | /         a  # ty:ignore[division-by-zero]
+        4 | |         /
+        5 | |         0
+          | |_________^
+        6 |   )
           |
         1 | 
-        2 |             b = (
-          -                     a  # ty:ignore[division-by-zero]
-        3 +                     a  # ty:ignore[division-by-zero, unresolved-reference]
-        4 |                     /
-        5 |                     0
-        6 |             )
+        2 | b = (
+          -         a  # ty:ignore[division-by-zero]
+        3 +         a  # ty:ignore[division-by-zero, unresolved-reference]
+        4 |         /
+        5 |         0
+        6 | )
         ");
     }
 
@@ -274,22 +271,21 @@ mod tests {
 
         assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @r"
         info[code-action]: Ignore 'unresolved-reference' for this line
-         --> main.py:3:21
+         --> main.py:3:9
           |
-        2 |               b = (
-        3 | /                     a
-        4 | |                     /
-        5 | |                     0  # ty:ignore[division-by-zero]
-          | |_____________________^
-        6 |               )
+        2 |   b = (
+        3 | /         a
+        4 | |         /
+        5 | |         0  # ty:ignore[division-by-zero]
+          | |_________^
+        6 |   )
           |
-        2 |             b = (
-        3 |                     a
-        4 |                     /
-          -                     0  # ty:ignore[division-by-zero]
-        5 +                     0  # ty:ignore[division-by-zero, unresolved-reference]
-        6 |             )
-        7 |
+        2 | b = (
+        3 |         a
+        4 |         /
+          -         0  # ty:ignore[division-by-zero]
+        5 +         0  # ty:ignore[division-by-zero, unresolved-reference]
+        6 | )
         ");
     }
 
@@ -307,22 +303,22 @@ mod tests {
 
         assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @r"
         info[code-action]: Ignore 'unresolved-reference' for this line
-         --> main.py:3:21
+         --> main.py:3:9
           |
-        2 |               b = (
-        3 | /                     a  # ty:ignore[division-by-zero]
-        4 | |                     /
-        5 | |                     0  # ty:ignore[division-by-zero]
-          | |_____________________^
-        6 |               )
+        2 |   b = (
+        3 | /         a  # ty:ignore[division-by-zero]
+        4 | |         /
+        5 | |         0  # ty:ignore[division-by-zero]
+          | |_________^
+        6 |   )
           |
         1 | 
-        2 |             b = (
-          -                     a  # ty:ignore[division-by-zero]
-        3 +                     a  # ty:ignore[division-by-zero, unresolved-reference]
-        4 |                     /
-        5 |                     0  # ty:ignore[division-by-zero]
-        6 |             )
+        2 | b = (
+          -         a  # ty:ignore[division-by-zero]
+        3 +         a  # ty:ignore[division-by-zero, unresolved-reference]
+        4 |         /
+        5 |         0  # ty:ignore[division-by-zero]
+        6 | )
         ");
     }
 
@@ -339,20 +335,19 @@ mod tests {
 
         assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @r#"
         info[code-action]: Ignore 'unresolved-reference' for this line
-         --> main.py:3:18
+         --> main.py:3:6
           |
-        2 |             b = f"""
-        3 |                 {a}
-          |                  ^
-        4 |                 more text
-        5 |             """
+        2 | b = f"""
+        3 |     {a}
+          |      ^
+        4 |     more text
+        5 | """
           |
-        2 |             b = f"""
-        3 |                 {a}
-        4 |                 more text
-          -             """
-        5 +             """  # ty:ignore[unresolved-reference]
-        6 |
+        2 | b = f"""
+        3 |     {a}
+        4 |     more text
+          - """
+        5 + """  # ty:ignore[unresolved-reference]
         "#);
     }
 
@@ -371,23 +366,23 @@ mod tests {
 
         assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @r#"
         info[code-action]: Ignore 'unresolved-reference' for this line
-         --> main.py:4:17
+         --> main.py:4:5
           |
-        2 |             b = f"""
-        3 |                 {
-        4 |                 a
-          |                 ^
-        5 |                 }
-        6 |                 more text
+        2 | b = f"""
+        3 |     {
+        4 |     a
+          |     ^
+        5 |     }
+        6 |     more text
           |
         1 | 
-        2 |             b = f"""
-        3 |                 {
-          -                 a
-        4 +                 a  # ty:ignore[unresolved-reference]
-        5 |                 }
-        6 |                 more text
-        7 |             """
+        2 | b = f"""
+        3 |     {
+          -     a
+        4 +     a  # ty:ignore[unresolved-reference]
+        5 |     }
+        6 |     more text
+        7 | """
         "#);
     }
 
@@ -403,19 +398,18 @@ mod tests {
 
         assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @r#"
         info[code-action]: Ignore 'unresolved-reference' for this line
-         --> main.py:2:17
+         --> main.py:2:5
           |
-        2 |             b = a + """
-          |                 ^
-        3 |                 more text
-        4 |             """
+        2 | b = a + """
+          |     ^
+        3 |     more text
+        4 | """
           |
         1 | 
-        2 |             b = a + """
-        3 |                 more text
-          -             """
-        4 +             """  # ty:ignore[unresolved-reference]
-        5 |
+        2 | b = a + """
+        3 |     more text
+          - """
+        4 + """  # ty:ignore[unresolved-reference]
         "#);
     }
 
@@ -430,17 +424,16 @@ mod tests {
 
         assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @r#"
         info[code-action]: Ignore 'unresolved-reference' for this line
-         --> main.py:2:17
+         --> main.py:2:5
           |
-        2 |             b = a \
-          |                 ^
-        3 |             + "test"
+        2 | b = a \
+          |     ^
+        3 | + "test"
           |
         1 | 
-        2 |             b = a \
-          -             + "test"
-        3 +             + "test"  # ty:ignore[unresolved-reference]
-        4 |
+        2 | b = a \
+          - + "test"
+        3 + + "test"  # ty:ignore[unresolved-reference]
         "#);
     }
 
@@ -454,27 +447,249 @@ mod tests {
 
         assert_snapshot!(test.code_actions(&UNDEFINED_REVEAL), @r"
         info[code-action]: import typing.reveal_type
-         --> main.py:2:13
+         --> main.py:2:1
           |
-        2 |             reveal_type(1)
-          |             ^^^^^^^^^^^
+        2 | reveal_type(1)
+          | ^^^^^^^^^^^
           |
         help: This is a preferred code action
         1 + from typing import reveal_type
         2 | 
-        3 |             reveal_type(1)
-        4 |         
+        3 | reveal_type(1)
 
         info[code-action]: Ignore 'undefined-reveal' for this line
-         --> main.py:2:13
+         --> main.py:2:1
           |
-        2 |             reveal_type(1)
-          |             ^^^^^^^^^^^
+        2 | reveal_type(1)
+          | ^^^^^^^^^^^
           |
         1 | 
-          -             reveal_type(1)
-        2 +             reveal_type(1)  # ty:ignore[undefined-reveal]
+          - reveal_type(1)
+        2 + reveal_type(1)  # ty:ignore[undefined-reveal]
+        ");
+    }
+
+    #[test]
+    fn unresolved_deprecated() {
+        let test = CodeActionTest::with_source(
+            r#"
+            @<START>deprecated<END>("do not use")
+            def my_func(): ...
+        "#,
+        );
+
+        assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @r#"
+        info[code-action]: import warnings.deprecated
+         --> main.py:2:2
+          |
+        2 | @deprecated("do not use")
+          |  ^^^^^^^^^^
+        3 | def my_func(): ...
+          |
+        help: This is a preferred code action
+        1 + from warnings import deprecated
+        2 | 
+        3 | @deprecated("do not use")
+        4 | def my_func(): ...
+
+        info[code-action]: Ignore 'unresolved-reference' for this line
+         --> main.py:2:2
+          |
+        2 | @deprecated("do not use")
+          |  ^^^^^^^^^^
+        3 | def my_func(): ...
+          |
+        1 | 
+          - @deprecated("do not use")
+        2 + @deprecated("do not use")  # ty:ignore[unresolved-reference]
+        3 | def my_func(): ...
+        "#);
+    }
+
+    #[test]
+    fn unresolved_deprecated_warnings_imported() {
+        let test = CodeActionTest::with_source(
+            r#"
+            import warnings
+
+            @<START>deprecated<END>("do not use")
+            def my_func(): ...
+        "#,
+        );
+
+        assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @r#"
+        info[code-action]: import warnings.deprecated
+         --> main.py:4:2
+          |
+        2 | import warnings
         3 |
+        4 | @deprecated("do not use")
+          |  ^^^^^^^^^^
+        5 | def my_func(): ...
+          |
+        help: This is a preferred code action
+        1 + from warnings import deprecated
+        2 | 
+        3 | import warnings
+        4 | 
+
+        info[code-action]: qualify warnings.deprecated
+         --> main.py:4:2
+          |
+        2 | import warnings
+        3 |
+        4 | @deprecated("do not use")
+          |  ^^^^^^^^^^
+        5 | def my_func(): ...
+          |
+        help: This is a preferred code action
+        1 | 
+        2 | import warnings
+        3 | 
+          - @deprecated("do not use")
+        4 + @warnings.deprecated("do not use")
+        5 | def my_func(): ...
+
+        info[code-action]: Ignore 'unresolved-reference' for this line
+         --> main.py:4:2
+          |
+        2 | import warnings
+        3 |
+        4 | @deprecated("do not use")
+          |  ^^^^^^^^^^
+        5 | def my_func(): ...
+          |
+        1 | 
+        2 | import warnings
+        3 | 
+          - @deprecated("do not use")
+        4 + @deprecated("do not use")  # ty:ignore[unresolved-reference]
+        5 | def my_func(): ...
+        "#);
+    }
+
+    // using `importlib.abc.ExecutionLoader` when no imports are in scope
+    #[test]
+    fn unresolved_loader() {
+        let test = CodeActionTest::with_source(
+            r#"
+            <START>ExecutionLoader<END>
+        "#,
+        );
+
+        assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @r"
+        info[code-action]: import importlib.abc.ExecutionLoader
+         --> main.py:2:1
+          |
+        2 | ExecutionLoader
+          | ^^^^^^^^^^^^^^^
+          |
+        help: This is a preferred code action
+        1 + from importlib.abc import ExecutionLoader
+        2 | 
+        3 | ExecutionLoader
+
+        info[code-action]: Ignore 'unresolved-reference' for this line
+         --> main.py:2:1
+          |
+        2 | ExecutionLoader
+          | ^^^^^^^^^^^^^^^
+          |
+        1 | 
+          - ExecutionLoader
+        2 + ExecutionLoader  # ty:ignore[unresolved-reference]
+        ");
+    }
+
+    // using `importlib.abc.ExecutionLoader` when `import importlib` is in scope
+    //
+    // TODO: `importlib.abc` is available whenever `importlib` is, so qualifying
+    // `importlib.abc.ExecutionLoader` without adding imports is actually legal here!
+    #[test]
+    fn unresolved_loader_importlib_imported() {
+        let test = CodeActionTest::with_source(
+            r#"
+            import importlib
+            <START>ExecutionLoader<END>
+        "#,
+        );
+
+        assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @r"
+        info[code-action]: import importlib.abc.ExecutionLoader
+         --> main.py:3:1
+          |
+        2 | import importlib
+        3 | ExecutionLoader
+          | ^^^^^^^^^^^^^^^
+          |
+        help: This is a preferred code action
+        1 + from importlib.abc import ExecutionLoader
+        2 | 
+        3 | import importlib
+        4 | ExecutionLoader
+
+        info[code-action]: Ignore 'unresolved-reference' for this line
+         --> main.py:3:1
+          |
+        2 | import importlib
+        3 | ExecutionLoader
+          | ^^^^^^^^^^^^^^^
+          |
+        1 | 
+        2 | import importlib
+          - ExecutionLoader
+        3 + ExecutionLoader  # ty:ignore[unresolved-reference]
+        ");
+    }
+
+    // Using `importlib.abc.ExecutionLoader` when `import importlib.abc` is in scope
+    #[test]
+    fn unresolved_loader_abc_imported() {
+        let test = CodeActionTest::with_source(
+            r#"
+            import importlib.abc
+            <START>ExecutionLoader<END>
+        "#,
+        );
+
+        assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @r"
+        info[code-action]: import importlib.abc.ExecutionLoader
+         --> main.py:3:1
+          |
+        2 | import importlib.abc
+        3 | ExecutionLoader
+          | ^^^^^^^^^^^^^^^
+          |
+        help: This is a preferred code action
+        1 + from importlib.abc import ExecutionLoader
+        2 | 
+        3 | import importlib.abc
+        4 | ExecutionLoader
+
+        info[code-action]: qualify importlib.abc.ExecutionLoader
+         --> main.py:3:1
+          |
+        2 | import importlib.abc
+        3 | ExecutionLoader
+          | ^^^^^^^^^^^^^^^
+          |
+        help: This is a preferred code action
+        1 | 
+        2 | import importlib.abc
+          - ExecutionLoader
+        3 + importlib.abc.ExecutionLoader
+
+        info[code-action]: Ignore 'unresolved-reference' for this line
+         --> main.py:3:1
+          |
+        2 | import importlib.abc
+        3 | ExecutionLoader
+          | ^^^^^^^^^^^^^^^
+          |
+        1 | 
+        2 | import importlib.abc
+          - ExecutionLoader
+        3 + ExecutionLoader  # ty:ignore[unresolved-reference]
         ");
     }
 
@@ -493,7 +708,7 @@ mod tests {
 
             db.init_program().unwrap();
 
-            let mut cleansed = source.to_string();
+            let mut cleansed = dedent(source).to_string();
 
             let start = cleansed
                 .find("<START>")
